-- Prep module
local M = {}
local ugfxCalled = 0

----------------------
-- Reset everything --
----------------------
local function reset()
	ugfxCalled = 0
	
	-- Log event
	log("D", "HighBeam9000_NavalBase", "reset called")
end

------------------
-- Get map name --
------------------
local function GetMapName()

	-- Get mission filename
	local missionFile = getMissionFilename()
	if missionFile == nil then
		log("E", "HighBeam9000_NavalBase", "No mission filename!")
		return nil
	end
	
	-- Split filename into path, file, ext
	local missionDir, missionName, missionExt = path.split(missionFile)
	if missionDir == nil then
		log("E", "HighBeam9000_NavalBase", "No mission directory!")
		return nil
	end
	
	-- Get map name
	local mapName = missionDir:sub(9, -2)
	if mapName == nil then
		log("E", "HighBeam9000_NavalBase", "No map name!")
		return nil
	end

	-- Return map name
	return mapName
end

-----------------------------
-- Spawn naval base object --
-----------------------------
-- scripts_HighBeam9000__NavalBase_extension.SpawnNavalBaseObject
local function SpawnNavalBaseObject(objType, objX, objY, objZ, objPhi, reCol)
	
	-- Check for valid type
	if objType ~= "Runway"
	and objType ~= "SimpleTarmac"
	and objType ~= "ThruHangar"
	and objType ~= "BasicRamp"
	and objType ~= "FlatPath"
	and objType ~= "ControlTower"
	and objType ~= "BridgeSpan"
	and objType ~= "SmallDock" then
		log("E", "HighBeam9000_NavalBase", "\"" .. objType .. "\" is not a valid naval base object type!")
		return
	end
	
	-- Get map name & object file
	local mapName = GetMapName()
	if mapName == nil then return end
	local objFile = "/levels/" .. mapName .."/art/NavalBase_" .. objType .. ".dae"
	
	-- Spawn TSStatic
	local nbObj = createObject('TSStatic')
	if nbObj == nil then
		log("E", "HighBeam9000_NavalBase", "Cannot create TSStatic!")
		return
	end
	
	-- Set as specified object type
	nbObj:setField('shapeName', 0, objFile)
	
	-- Set position, rotation and scale
	local quat = quatFromEuler(0, 0, objPhi / 180 * math.pi)
	nbObj:setPosRot(objX, objY, objZ, quat.x, quat.y, quat.z, quat.w)
	--nbObj.scale = vec3(1, 1, 1)

	-- Set collision map stuff
	nbObj:setField('collisionType', 0, "Visible Mesh Final")
	nbObj:setField('decalType', 0, "Visible Mesh Final")

	-- Register object
	nbObj.canSave = true
	nbObj:registerObject('')
	scenetree.MissionGroup:add(nbObj)
	
	-- One more collision thing
	if(reCol == nil or reCol == 1) then
		be:reloadStaticCollision(false)
	end
end

----------------------------
-- Spawn Fjord Naval Base --
----------------------------
-- scripts_HighBeam9000__NavalBase_extension.SpawnFjordNavalBase(baseStyle)
local function SpawnFjordNavalBase(baseStyle)
	
	-- Original base style
	if baseStyle == 0 then
		SpawnNavalBaseObject("Runway", -261.988, 8226.790, 59.000, -65.000, 0)
		SpawnNavalBaseObject("SimpleTarmac", -325.211, 8091.404, 59.000, -65.000, 0)
		SpawnNavalBaseObject("ThruHangar", -325.211, 8091.404, 179.000, 25.000, 0)
		SpawnNavalBaseObject("ControlTower", -302.444, 8141.478, 159.000, 25.000, 0)
		
		SpawnNavalBaseObject("SmallDock", 55.616, 7906.951, 89.000, -110.000, 0)
		SpawnNavalBaseObject("SmallDock", 83.319, 7886.292, 89.000, -110.000, 0)
		SpawnNavalBaseObject("SmallDock", 110.752, 7864.864, 89.000, -110.000, 0)
		SpawnNavalBaseObject("SmallDock", 138.237, 7844.115, 89.000, -110.000, 0)
		
		SpawnNavalBaseObject("BasicRamp", -98.073, 8034.701, 149.000, 25.000, 0)
		SpawnNavalBaseObject("BasicRamp", 12.556, 7928.241, 139.000, 40.000, 0)
		
		SpawnNavalBaseObject("FlatPath", -41.035, 7985.706, 49.000, 141.163, 0)
		SpawnNavalBaseObject("FlatPath", 175.829, 7799.868, 39.000, 127.935, 0)
		SpawnNavalBaseObject("FlatPath", 253.539, 7739.258, 39.000, 127.935, 0)
		be:reloadStaticCollision(false)
		
	-- New base style
	elseif baseStyle == 1 then
		SpawnNavalBaseObject("Runway", -851.595, 8333.228, 89, 115, 0)
		SpawnNavalBaseObject("SimpleTarmac", -788.234, 8468.806, 89, 115, 0)
		SpawnNavalBaseObject("ThruHangar", -788.234, 8468.806, 209, 25, 0)
		SpawnNavalBaseObject("ControlTower", -596.416, 8425.247, 189, 25, 0)
		
		SpawnNavalBaseObject("SmallDock", -366.036, 8250.086, 89, -155, 0)
		SpawnNavalBaseObject("SmallDock", -339.001, 8237.477, 89, -155, 0)
		SpawnNavalBaseObject("SmallDock", -285.230, 8212.377, 89, -155, 0)
		SpawnNavalBaseObject("SmallDock", -312.182, 8224.924, 89, -155, 0)
		
		SpawnNavalBaseObject("BasicRamp", -598.888, 8331.083, 179, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -553.534, 8309.934, 169, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -507.949, 8288.637, 159, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -462.478, 8267.420, 149, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -416.984, 8246.209, 139, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -406.326, 8252.128, 137.51, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -535.488, 8523.556, 179, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -489.852, 8502.243, 169, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -476.665, 8507.052, 159, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -522.037, 8528.226, 149, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -528.925, 8531.503, 147.51, -155	, 0)
		SpawnNavalBaseObject("BasicRamp", -913.969, 8700.284, 179, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -959.284, 8721.371, 169, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -964.196, 8734.610, 159, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -918.942, 8713.491, 149, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -911.908, 8710.217, 147.51, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -488.304, 8624.657, 179, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -443.263, 8603.623, 169, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -475.351, 8629.584, 149, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -482.135, 8632.751, 147.51, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -866.989, 8801.109, 179, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -912.310, 8822.251, 169, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -917.079, 8835.544, 159, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -871.719, 8814.381, 149, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -865.157, 8811.354, 147.51, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -430.064, 8608.454, 159, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -1060.460, 8629.030, 149, -65, 0)
		SpawnNavalBaseObject("BasicRamp", -1081.630, 8583.651, 159, -65, 0)
		SpawnNavalBaseObject("BasicRamp", -1015.427, 8525.228, 179, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -1060.727, 8546.334, 169, -155, 0)
		SpawnNavalBaseObject("BasicRamp", -1027.358, 8640.843, 140.821, 25, 0)
		SpawnNavalBaseObject("BasicRamp", -1035.839, 8617.183, 137.432, 115, 0)
--		SpawnNavalBaseObject("BasicRamp", -1027.080, 8536.005, 139, 0, 0)
--		SpawnNavalBaseObject("BasicRamp", -1079.789, 8599.451, 119, 0, 0)
--		SpawnNavalBaseObject("BasicRamp", -1096.773, 8562.933, 119, 0, 0)
		
		SpawnNavalBaseObject("FlatPath", -620.244, 8450.995, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -580.340, 8322.173, 69, -65, 0)
		SpawnNavalBaseObject("FlatPath", -602.545, 8382.480, 47.51, 25, 0)
		SpawnNavalBaseObject("FlatPath", -710.677, 8493.117, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -801.215, 8535.338, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -891.701, 8577.582, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -543.340, 8384.295, 47.51, -58.5, 0)
		SpawnNavalBaseObject("FlatPath", -502.377, 8359.201, 47.51, -58.5, 0)
		SpawnNavalBaseObject("FlatPath", -447.807, 8287.967, 47.51, -16.4, 0)
		SpawnNavalBaseObject("FlatPath", -478.687, 8285.882, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -483.580, 8499.312, 69, -65, 0)
		SpawnNavalBaseObject("FlatPath", -494.892, 8515.456, 49, -65, 0)
		SpawnNavalBaseObject("FlatPath", -584.642, 8557.480, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -674.992, 8599.637, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -765.353, 8641.777, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -965.467, 8724.225, 69, -65, 0)
		SpawnNavalBaseObject("FlatPath", -946.008, 8726.059, 49, -65, 0)
		SpawnNavalBaseObject("FlatPath", -855.225, 8683.695, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -433.401, 8598.986, 69, -65, 0)
		SpawnNavalBaseObject("FlatPath", -448.290, 8616.903, 49, -65, 0)
		SpawnNavalBaseObject("FlatPath", -538.697, 8659.127, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -628.943, 8701.199, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -719.319, 8743.349, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -923.153, 8827.307, 69, -65, 0)
		SpawnNavalBaseObject("FlatPath", -898.934, 8827.090, 49, -65, 0)
		SpawnNavalBaseObject("FlatPath", -809.767, 8785.544, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -982.127, 8619.749, 47.51, -65, 0)
		SpawnNavalBaseObject("FlatPath", -1068.975, 8610.649, 39, 25, 0)
		
		SpawnNavalBaseObject("BridgeSpan", -603.216, 8437.392, 89, -65, 0)
		SpawnNavalBaseObject("BridgeSpan", -560.195, 8529.668, 89, 115, 0)
		SpawnNavalBaseObject("BridgeSpan", -556.061, 8538.482, 89, -65, 0)
		SpawnNavalBaseObject("BridgeSpan", -513.047, 8630.755, 89, 115, 0)
		SpawnNavalBaseObject("BridgeSpan", -936.380, 8592.989, 89, -65, 0)
		SpawnNavalBaseObject("BridgeSpan", -893.364, 8685.265, 89, 115, 0)
		SpawnNavalBaseObject("BridgeSpan", -889.343, 8693.960, 89, -65, 0)
		SpawnNavalBaseObject("BridgeSpan", -846.397, 8786.077, 89, 115, 0)
		be:reloadStaticCollision(false)
	else
		log("E", "HighBeam9000_NavalBase", tostring(baseStyle) .. " is not a valid base style!")
	end
end

------------------------------
-- Perform timed operations --
------------------------------
local function onUpdate(dtReal, dtSim, dtRaw)
	
	-- Log event
	if ugfxCalled == 0 then
		ugfxCalled = 1
		log("D", "HighBeam9000_NavalBase", "onUpdate called")
	end
end

----------------------
-- Public interface --
----------------------
M.SpawnFjordNavalBase = SpawnFjordNavalBase
M.SpawnNavalBaseObject = SpawnNavalBaseObject
M.onExtensionLoaded = reset
M.onExtensionUnloaded = reset
M.reset = reset
M.onUpdate = onUpdate
return M
